@@ -14,7 +14,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-#  $Id: ninix-install.py,v 1.39 2006/02/02 14:06:31 shy Exp $
+#  $Id: ninix-install.py,v 1.40 2006/08/26 16:01:17 shy Exp $
 #
 
 # env.var: NINIX_HOME, NINIX_USER, NINIX_ARCHIVE, TMPDIR
@@ -66,8 +66,7 @@ Options:
 '''
 
 def usage():
-    sys.stderr.write(USAGE % PROGRAM_NAME)
-    sys.exit(1)
+    raise SystemExit, USAGE % PROGRAM_NAME
 
 
 class InstallError(Exception):
@@ -403,7 +402,7 @@ def confirm_overwrite(path):
     except EOFError:
         answer = None
     except KeyboardInterrupt:
-        sys.exit(1)
+        raise SystemExit
     if not answer:
         return 0
     return answer.lower().startswith('y')
@@ -417,7 +416,7 @@ def confirm_removal(path):
     except EOFError:
         answer = None
     except KeyboardInterrupt:
-        sys.exit(1)
+        raise SystemExit
     if not answer:
         return 0
     return answer.lower().startswith('y')
