@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -14,6 +14,7 @@
 #
 
 import os
+import gc
 
 if 'DISPLAY' in os.environ:
     import gtk
@@ -434,6 +435,8 @@ class BalloonWindow:
         self.clear_text()
 
     def reset_pixbuf_cache(self):
+        self.pixbuf = None
+        gc.collect()
         self.pixbuf = {}
         for key, (path, config) in self.balloon.iteritems():
             try:
@@ -501,18 +504,18 @@ class BalloonWindow:
             self.sstp.append((x, y)) # sstp message
         # arrow pixmaps and masks
         x, y = self.arrow[0]
-        pixbuf = self.pixbuf['arrow0'][0]
+        pixbuf, (w, h) = self.pixbuf['arrow0']
         scale = self.get_scale()
-        w = max(1, int(pixbuf.get_width() * scale / 100))
-        h = max(1, int(pixbuf.get_height() * scale / 100))
+        w = max(1, int(w * scale / 100))
+        h = max(1, int(h * scale / 100))
         pixmap, mask = pixbuf.scale_simple(
             w, h, gtk.gdk.INTERP_BILINEAR).render_pixmap_and_mask(255)
         self.arrow0_pixmap = pixmap, mask, (w, h)
         self.arrow0_gc = self.new_mask_gc(mask, x, y)
         x, y = self.arrow[1]
-        pixbuf = self.pixbuf['arrow1'][0]
-        w = max(1, int(pixbuf.get_width() * scale / 100))
-        h = max(1, int(pixbuf.get_height() * scale / 100))
+        pixbuf, (w, h) = self.pixbuf['arrow1']
+        w = max(1, int(w * scale / 100))
+        h = max(1, int(h * scale / 100))
         pixmap, mask = pixbuf.scale_simple(
             w, h, gtk.gdk.INTERP_BILINEAR).render_pixmap_and_mask(255)
         self.arrow1_pixmap = pixmap, mask, (w, h)
@@ -520,9 +523,9 @@ class BalloonWindow:
         # sstp marker pixmap and mask
         if self.side == 0 and 'sstp' in self.pixbuf:
             x, y = self.sstp[0]
-            pixbuf = self.pixbuf['sstp'][0]
-            w = max(1, int(pixbuf.get_width() * scale / 100))
-            h = max(1, int(pixbuf.get_height() * scale / 100))
+            pixbuf, (w, h) = self.pixbuf['sstp']
+            w = max(1, int(w * scale / 100))
+            h = max(1, int(h * scale / 100))
             pixmap, mask = pixbuf.scale_simple(
                 w, h, gtk.gdk.INTERP_BILINEAR).render_pixmap_and_mask(255)
             self.sstp_pixmap = pixmap, mask, (w, h)
@@ -764,9 +767,8 @@ class BalloonWindow:
             x, y = self.sstp[0]
             w, h = self.sstp_pixmap[2]
             if self.sstp_message:
-                pixmap = self.sstp_pixmap[0]
                 self.darea.window.draw_drawable(
-                    self.sstp_gc, pixmap, 0, 0, x, y, w, h)
+                    self.sstp_gc, self.sstp_pixmap[0], 0, 0, x, y, w, h)
             else:
                 self.redraw_area(x, y, w, h)
         # draw/erase sstp message
@@ -780,20 +782,20 @@ class BalloonWindow:
 
     def redraw_arrow0(self, clear=1):
         x, y = self.arrow[0]
-        pixmap, mask, (w, h) = self.arrow0_pixmap
+        w, h = self.arrow0_pixmap[2]
         if self.lineno > 0:
             self.darea.window.draw_drawable(
-                self.arrow0_gc, pixmap, 0, 0, x, y, w, h)
+                self.arrow0_gc, self.arrow0_pixmap[0], 0, 0, x, y, w, h)
         elif clear:
             self.redraw_area(x, y, w, h)
 
     def redraw_arrow1(self, clear=1):
         x, y = self.arrow[1]
-        pixmap, mask, (w, h) = self.arrow1_pixmap
+        w, h = self.arrow1_pixmap[2]
         if self.lineno + self.lines < len(self.text_buffer) or \
            self.sakura.script_mode == self.sakura.PAUSE_MODE:
             self.darea.window.draw_drawable(
-                self.arrow1_gc, pixmap, 0, 0, x, y, w, h)
+                self.arrow1_gc, self.arrow1_pixmap[0], 0, 0, x, y, w, h)
         elif clear:
             self.redraw_area(x, y, w, h)
 
@@ -895,15 +897,15 @@ class BalloonWindow:
                                          x, y, self.layout)
             for l, c in self.sstp_marker:
                 if l == i:
-                    pixmap, mask, (mw, mh) = self.sstp_pixmap
+                    mw, mh = self.sstp_pixmap[2]
                     self.layout.set_text(self.text_buffer[i][:c])
                     text_w, text_h = self.layout.get_pixel_size()
                     mx = x + text_w
                     my = y + (self.font_height + self.line_space) / 2
                     my = my - mh / 2
-                    gc = self.new_mask_gc(mask, mx, my)
+                    gc = self.new_mask_gc(self.sstp_pixmap[1], mx, my)
                     self.darea.window.draw_drawable(
-                        gc, pixmap, 0, 0, mx, my, mw, mh)
+                        gc, self.sstp_pixmap[0], 0, 0, mx, my, mw, mh)
             i += 1
             line += 1
         if self.side == 0 and self.sstp_message:
@@ -1229,6 +1231,8 @@ class BalloonWindow:
         w = pixbuf.get_width()
         h = pixbuf.get_height()
         self.images.append((pixbuf, (w, h), (x, y)))
+        del pixbuf
+        gc.collect()
         self.redraw()
 
     def draw_last_line(self, column=0):
@@ -1245,15 +1249,15 @@ class BalloonWindow:
                 self.darea.queue_draw_area(x, y, w, h)
             for l, c in self.sstp_marker:
                 if l == line:
-                    pixmap, mask, (mw, mh) = self.sstp_pixmap
+                    mw, mh = self.sstp_pixmap[2]
                     self.layout.set_text(self.text_buffer[l][:c])
                     text_w, text_h = self.layout.get_pixel_size()
                     mx = x + text_w
                     my = y + (self.font_height + self.line_space) / 2
                     my = my - mh / 2
-                    gc = self.new_mask_gc(mask, mx, my)
+                    gc = self.new_mask_gc(self.sstp_pixmap[1], mx, my)
                     self.darea.window.draw_drawable(
-                        gc, pixmap, 0, 0, mx, my, mw, mh)
+                        gc, self.sstp_pixmap[0], 0, 0, mx, my, mw, mh)
         else:
             self.redraw()
             while line >= self.lineno + self.lines:
