@@ -1,7 +1,7 @@
 # -*- coding: ascii -*-
 #
 #  bln.py - a easyballoon compatible Saori module for ninix
-#  Copyright (C) 2002-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -20,6 +20,7 @@ import random
 import math
 import time
 import sys
+import gc
 
 if 'DISPLAY' in os.environ:
     if 'gtk' not in sys.modules:
@@ -238,7 +239,9 @@ class Balloon:
             h = max(1, int(h * self.scale / 100))
             balloon_pixbuf = balloon_pixbuf.scale_simple(
                 w, h, gtk.gdk.INTERP_BILINEAR)
-        balloon_pixmap, self.mask_pixmap = balloon_pixbuf.render_pixmap_and_mask(1)
+        balloon_pixmap, mask_pixmap = balloon_pixbuf.render_pixmap_and_mask(1)
+        del balloon_pixbuf
+        gc.collect()
         if self.position == 'lefttop':
             pass
         elif self.position == 'leftbottom':
@@ -311,7 +314,7 @@ class Balloon:
             self.top = int(int(data['disparea.top']) * self.scale / 100)
         if 'disparea.bottom' in data:
             self.bottom = int(int(data['disparea.bottom']) * self.scale / 100)
-        self.window.shape_combine_mask(self.mask_pixmap, 0, 0)
+        self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.script = None
         self.darea = gtk.DrawingArea()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
