@@ -1,7 +1,7 @@
 # -*- coding: UTF-8 -*-
 #
 #  hanayu.py - a "花柚" compatible Saori module for ninix
-#  Copyright (C) 2002-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -11,7 +11,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-#  $Id: hanayu.py,v 1.32 2006/03/25 08:07:09 shy Exp $
+#  $Id: hanayu.py,v 1.34 2007/04/10 14:45:38 shy Exp $
 #
 
 # TODO:
@@ -22,6 +22,7 @@ import sys
 import os
 import math
 import time
+import gc
 
 if 'DISPLAY' in os.environ:
     if 'gtk' not in sys.modules:
@@ -302,6 +303,8 @@ class Graph:
                                             0, 0, xoffset, yoffset,
                                             width, height,
                                             gtk.gdk.RGB_DITHER_NONE, 0, 0)
+            del pixbuf
+            gc.collect()
         self.darea.window.draw_drawable(self.surface_gc, self.surface_pixmap,
                                         0, 0, 0, 0, self.width, self.height)
 
