@@ -13,7 +13,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-#  $Id: kawari.py,v 1.30 2005/10/06 02:59:13 shy Exp $
+#  $Id: kawari.py,v 1.31 2006/08/26 16:01:18 shy Exp $
 #
 
 import __builtin__
@@ -1961,8 +1961,7 @@ def test():
     try:
         options, argv = getopt.getopt(sys.argv[1:], 'd:', ['debug='])
     except getopt.error, e:
-        sys.stderr.write(''.join((str(e), '\n')))
-        sys.exit(1)
+        raise SystemExit, ''.join((str(e), '\n'))
     if argv:
         kawari_dir = argv[0]
     else:
