@@ -12,7 +12,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: misaka.py,v 1.28 2006/05/24 18:55:18 shy Exp $
+# $Id: misaka.py,v 1.29 2006/09/03 15:37:06 shy Exp $
 #
 
 # TODO:
@@ -28,6 +28,11 @@ import sys
 import time
 import random
 
+try:
+    import chardet.universaldetector
+except:
+    chardet = None
+
 
 class MisakaError(ValueError):
     pass
@@ -278,8 +283,9 @@ NODE_POW_EXPR    = 14
 
 class Parser:
 
-    def __init__(self, debug=0):
-        self.lexer = Lexer(debug=debug)
+    def __init__(self, charset, debug=0):
+        self.lexer = Lexer(charset=charset, debug=debug)
+        self.charset = charset
         self.debug = debug
 
     def read(self, f, path=None):
@@ -1045,6 +1051,30 @@ class Shiori:
         self.misaka_error = error
         global_variables = []
         global_constants = []
+        # charset auto-detection
+        if chardet is not None:
+            detector = chardet.universaldetector.UniversalDetector()
+            for filename in filelist:
+                path = os.path.join(self.misaka_dir, filename)
+                try:
+                    f = __builtin__.open(path)
+                except IOError:
+                    if self.debug & 4:
+                        print 'cannot read', filename
+                    continue
+                basename, suffix = os.path.splitext(filename)
+                if suffix == '.__1':
+                    f = StringIO.StringIO(self.crypt(f.read()))
+                detector.reset()
+                for line in f:
+                    detector.feed(line)
+                    if detector.done:
+                        break
+                else:
+                    detector.close()
+                    if detector.result['confidence'] > 0.98: # XXX
+                        self.charset = detector.result['encoding']
+                        break
         for filename in filelist:
             path = os.path.join(self.misaka_dir, filename)
             try:
@@ -1083,7 +1113,7 @@ class Shiori:
                     self.eval_function(node[1], node[2])
 
     def read(self, f, path=None):
-        parser = Parser(self.debug)
+        parser = Parser(self.charset, self.debug)
         parser.read(f, path)
         common, dic = parser.get_dict()
         variables = []
@@ -2170,8 +2200,8 @@ def test_ini(dirlist):
         print 'debug =', debug
         print 'error =', error
 
-def test_lexer(filelist):
-    lexer = Lexer(debug=1)
+def test_lexer(filelist, charset='Shift_JIS'):
+    lexer = Lexer(charset=charset, debug=1)
     for filename in filelist:
         print 'Reading', filename, '...'
         lexer.read(__builtin__.open(filename))
@@ -2203,10 +2233,10 @@ def test_lexer(filelist):
         else:
             print lexeme.encode('UTF-8', 'ignore')
 
-def test_parser(filelist):
+def test_parser(filelist, charset='Shift_JIS'):
     for filename in filelist:
         print 'Reading', filename, '...'
-        parser = Parser(debug=1)
+        parser = Parser(charset=charset, debug=1)
         parser.read(__builtin__.open(filename))
         common, dic = parser.get_dict()
         if common is not None:
@@ -2251,14 +2281,15 @@ def test_interpreter(top_dir):
 
 if __name__ == '__main__':
     USAGE= 'Usage: misaka.py [ini|lexer|parser|interp] ...'
+    charset = 'Shift_JIS' # XXX
     if len(sys.argv) == 1:
         print USAGE
     elif sys.argv[1] == 'ini':
         test_ini(sys.argv[2:])
     elif sys.argv[1] == 'lexer':
-        test_lexer(sys.argv[2:])
+        test_lexer(sys.argv[2:], charset)
     elif sys.argv[1] == 'parser':
-        test_parser(sys.argv[2:])
+        test_parser(sys.argv[2:], charset)
     elif sys.argv[1] == 'interp':
         test_interpreter(sys.argv[2])
     else:
