@@ -930,7 +930,7 @@ class Ghost:
                 if self.__temp_mode == 1:
                     time.sleep(1.4)
                     self.finalize()
-                    self.app.remove_ghost(self) ## FIXME
+                    self.app.close_ghost(self) ## FIXME
                     self.app.reset_sstp_flag() ## FIXME
                     return False
                 else:
