@@ -11,7 +11,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: home.py,v 1.31 2006/02/02 14:06:31 shy Exp $
+# $Id: home.py,v 1.32 2006/08/26 16:01:18 shy Exp $
 #
 
 import os
@@ -641,8 +641,7 @@ def test():
         pass
     config = load_config()
     if config is None:
-        sys.stderr.write('Home directory not found.\n')
-        sys.exit(1)
+        raise SystemExit, 'Home directory not found.\n'
     ghosts, shells, balloons, plugins, nekoninni, katochan, kinoko = config ## FIXME
     # ghosts
     for desc, shiori_dir, use_makoto, surface_set, balloon, prefix, shiori_dll, shiori_name in ghosts: ## FIXME
