@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 #
-#  Copyright (C) 2004-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -16,6 +16,7 @@
 # - 透明度の設定
 
 import os
+import gc
 
 if 'DISPLAY' in os.environ:
     import gtk
@@ -190,6 +191,7 @@ class Kinoko:
 class Skin:
 
     def __init__(self, data, accelgroup, kinoko, scale):
+        self.frame_buffer = []
         self.data = data
         self.accelgroup = accelgroup
         self.kinoko = kinoko
@@ -231,22 +233,21 @@ class Skin:
         path = os.path.join(self.data['dir'], self.data['base'])
         try:
             self.pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            surface_pixbuf = self.pixbuf.copy()
             if self.__scale != 100:
-                w = max(1, surface_pixbuf.get_width() * self.__scale / 100)
-                h = max(1, surface_pixbuf.get_height() * self.__scale / 100)
-                surface_pixbuf = surface_pixbuf.scale_simple(
+                w = max(1, self.pixbuf.get_width() * self.__scale / 100)
+                h = max(1, self.pixbuf.get_height() * self.__scale / 100)
+                surface_pixbuf = self.pixbuf.scale_simple(
                     w, h, gtk.gdk.INTERP_BILINEAR)
             image, mask = surface_pixbuf.render_pixmap_and_mask(255)
+            del surface_pixbuf
+            gc.collect()
         except: ## FIXME
             self.kinoko.close(None)
             return
         ## FIXME
         self.w, self.h = image.get_size()
         self.set_position()
-        self.surface_pixmap = image
-        self.mask_pixmap = mask
-        self.window.shape_combine_mask(self.mask_pixmap, 0, 0)
+        self.window.shape_combine_mask(mask, 0, 0)
         self.darea = gtk.DrawingArea()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
         self.darea.connect('expose_event', self.redraw)
@@ -305,13 +306,6 @@ class Skin:
     def redraw(self, widget, event): ## FIXME
         pass
 
-    def show_surface(self):
-        self.darea.window.set_back_pixmap(self.surface_pixmap, False)
-        self.window.shape_combine_mask(self.mask_pixmap, 0, 0)
-        #print 'DRAW:', dir(self.darea)
-        #self.darea.queue_draw_area(0, 0, w, h)
-        self.darea.queue_draw()
-
     def get_pixbuf(self, surface_id):
         path = os.path.join(self.data['dir'], ''.join(('surface', str(surface_id), '.png')))
         if os.path.exists(path):
@@ -327,9 +321,52 @@ class Skin:
             surface_pixbuf = self.pixbuf.copy()
         return surface_pixbuf
 
-    def update(self, cache=None):
-        if cache is not None:
-            surface_pixbuf = cache
+    def clear_frame_buffer(self):
+        self.frame_buffer = []
+
+    def update_frame_buffer(self, seriko, frame, move, dirty):
+        if dirty:
+            surface_pixbuf = self.create_surface_pixbuf(seriko.base_id)
+            if surface_pixbuf is not None:
+                # draw overlays
+                for pixbuf_id, x, y in seriko.iter_overlays():
+                    try:
+                        pixbuf = self.get_pixbuf(pixbuf_id)
+                        w = pixbuf.get_width()
+                        h = pixbuf.get_height()
+                    except:
+                        continue
+                    # overlay surface pixbuf
+                    sw = surface_pixbuf.get_width()
+                    sh = surface_pixbuf.get_height()
+                    if x + w > sw:
+                        w = sw - x
+                    if y + h > sh:
+                        h = sh - y
+                    if x < 0:
+                        dest_x = 0
+                        w += x
+                    else:
+                        dest_x = x
+                    if y < 0:
+                        dest_y = 0
+                        h += y
+                    else:
+                        dest_y = y
+                    pixbuf.composite(surface_pixbuf, dest_x, dest_y,
+                                     w, h, x, y, 1.0, 1.0,
+                                     gtk.gdk.INTERP_BILINEAR, 255)
+        else:
+            surface_pixbuf = None
+        self.frame_buffer.append((frame, move, surface_pixbuf, dirty))
+
+    def update(self):
+        if len(self.frame_buffer) > 0:
+            frame, move, surface_pixbuf, dirty = self.frame_buffer.pop(0)
+            if move is not None:
+                self.move_surface(*move)
+            if not dirty:
+                return
         else:
             surface_pixbuf = self.create_surface_pixbuf()
         if self.__scale != 100:
@@ -337,16 +374,21 @@ class Skin:
             h = max(1, surface_pixbuf.get_height() * self.__scale / 100)
             surface_pixbuf = surface_pixbuf.scale_simple(
                 w, h, gtk.gdk.INTERP_BILINEAR)
-        self.surface_pixmap, self.mask_pixmap = \
-                             surface_pixbuf.render_pixmap_and_mask(255)
+        image, mask = surface_pixbuf.render_pixmap_and_mask(255)
+        del surface_pixbuf
+        gc.collect()
         self.w, self.h = self.surface_pixmap.get_size()
-        self.show_surface()
+        self.darea.window.set_back_pixmap(image, False)
+        self.window.shape_combine_mask(mask, 0, 0)
+        #print 'DRAW:', dir(self.darea)
+        #self.darea.queue_draw_area(0, 0, w, h)
+        self.darea.queue_draw()
 
     def terminate(self):
-        self.seriko.terminate()
+        self.seriko.terminate(self)
 
     def add_overlay(self, actor, surface_id, x, y):
-        self.seriko.add_overlay(actor, surface_id, x, y)
+        self.seriko.add_overlay(self, actor, surface_id, x, y)
 
     def remove_overlay(self, actor):
         self.seriko.remove_overlay(actor)
@@ -365,7 +407,7 @@ class Skin:
         self.kinoko.close(None)
 
     def destroy(self): ## FIXME
-        self.seriko.terminate()
+        self.seriko.terminate(self)
         self.window.destroy()
 
     def button_press(self, widget, event): ## FIXME
