@@ -3,7 +3,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003-2005 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -13,7 +13,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: main.py,v 1.138 2006/10/05 15:16:44 shy Exp $
+# $Id: main.py,v 1.142 2007/04/29 14:25:30 shy Exp $
 #
 
 import getopt
@@ -26,6 +26,7 @@ import gettext
 import traceback
 import cStringIO
 import fcntl
+import gc
 
 gettext.install('ninix')
 
@@ -560,6 +561,19 @@ class Application:
                 icon_path = None
         else:
             icon_path = None
+        if icon_path is not None:
+            try:
+                pixbuf = ninix.pix.create_pixbuf_from_file(icon_path, is_pnr=0)
+            except:
+                pixbuf = None
+            if pixbuf is not None:
+                icon = pixbuf.scale_simple(16, 16, gtk.gdk.INTERP_BILINEAR)
+                del pixbuf
+                gc.collect()
+            else:
+                icon = None
+        else:
+            icon = None
         name = desc.get('name',
                         ''.join((unicode(_('Ghost'), 'utf-8'),
                                  '#%d' % (i + 1))))
@@ -570,7 +584,7 @@ class Application:
             shell_list.append((shell_name, value))
         item = {}
         item['name'] = name
-        item['icon'] = icon_path
+        item['icon'] = icon
         item['shell'] = shell_list
         item['instance'] = None
         return item
@@ -811,12 +825,16 @@ class Application:
             if working_list:
                 self.current_sakura = random.choice(working_list).current
 
-    def remove_ghost(self, ghost): ## FIXME
+    def close_ghost(self, ghost):
         if not self.get_working_ghost():
             self.quit()
         elif self.current_sakura == ghost.current:
             self.select_current_sakura()
 
+    def close_all_ghosts(self):
+        for ghost in self.get_working_ghost():
+            ghost.close()
+
     def quit(self):
         gobject.source_remove(self.timeout_id)
         self.usage_dialog.close()
@@ -901,7 +919,7 @@ class Application:
         if vanished:
             ghost.finalize()
             self.start_sakura(item, ghost.current, vanished)
-            self.remove_ghost(ghost)
+            self.close_ghost(ghost)
         elif not event:
             proc()
         elif ghost.current[0] == set_type and ghost.current[1] == i:
@@ -915,7 +933,7 @@ class Application:
         ghost.finalize()
         if starter is not None:
             starter(*args)
-        self.remove_ghost(ghost)
+        self.close_ghost(ghost)
 
     def request_shell(self, index):
         return self.shells[index]
