@@ -1,6 +1,6 @@
 # -*- coding: ascii -*-
 #
-#  Copyright (C) 2003-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -13,6 +13,7 @@
 
 import os
 import re
+import gc
 
 if 'DISPLAY' in os.environ:
     import gtk
@@ -30,42 +31,48 @@ class Menu:
         ui_info = '''
         <ui>
           <popup name='popup'>
-            <menu action='Portal'>
-            </menu>
             <menu action='Recommend'>
             </menu>
+            <menu action='Portal'>
+            </menu>
             <separator/>
-            <menu action='File'>
+            <menu action='Plugin'>
+            </menu>
+            <menuitem action='Scaling'/>
+            <menuitem action='Stick'/>
+            <menuitem action='Wait'/>
+            <separator/>
+            <menu action='Options'>
             <menuitem action='Update'/>
             <menuitem action='Vanish'/>
             <menuitem action='Reload'/>
-            <menuitem action='Settings'/>
+            <menuitem action='Preferences'/>
             <menuitem action='Manager'/>
-            <menuitem action='Usage'/>
-            <menuitem action='Version'/>
-            <menuitem action='Exit'/>
             </menu>
+            <separator/>
             <menu action='Change'>
             </menu>
             <menu action='Summon'>
             </menu>
             <menu action='Shell'>
             </menu>
+            <menu action='Costume'>
+            </menu>
             <menu action='Balloon'>
             </menu>
             <separator/>
-            <menuitem action='Scaling'/>
-            <menu action='Costume'>
+            <menu action='Information'>
+            <menuitem action='Usage'/>
+            <menuitem action='Version'/>
             </menu>
-            <menuitem action='Stick'/>
-            <menuitem action='Wait'/>
             <separator/>
             <menu action='Nekodorif'>
             </menu>
             <menu action='Kinoko'>
             </menu>
-            <menu action='Plugin'>
-            </menu>
+            <separator/>
+            <menuitem action='Close'/>
+            <menuitem action='Quit'/>
           </popup>
         </ui>
         '''
@@ -74,33 +81,38 @@ class Menu:
                        None, 1, 1],
             'Recommend': [('Recommend', None, _('Recommend sites(_R)'), None),
                           None, 1, 1],
-            'File': [('File', None, _('File(_F)'), None),
+            'Options': [('Options', None, _('Options(_F)'), None),
                      None, 1, 1],
-            'File/Update': [('Update', None, _('Network Update(_U)'), None,
+            'Options/Update': [('Update', None, _('Network Update(_U)'), None,
                              '', lambda *a: self.__ghost.network_update()),
                             None, 1, 1],
-            'File/Vanish': [('Vanish', None, _('Vanish(_F)'), None,
+            'Options/Vanish': [('Vanish', None, _('Vanish(_F)'), None,
                              '', lambda *a: self.__ghost.vanish()),
                             None, 1, 1],
-            'File/Reload': [('Reload', None, _('Reload(_L)'), None,
+            'Options/Reload': [('Reload', None, _('Reload(_L)'), None,
                              '', lambda *a: self.__ghost.reload()),
                             None, 1, 1],
-            'File/Settings': [('Settings', None, _('Settings...(_O)'), None,
+            'Options/Preferences': [('Preferences', None, _('Preferences...(_O)'), None,
                                '', lambda *a: self.__ghost.edit_preferences()),
                               None, 1, 1],
-            'File/Manager': [('Manager', None, _('Ghost Manager(_M)'), None,
+            'Options/Manager': [('Manager', None, _('Ghost Manager(_M)'), None,
                               '',
                               lambda *a: self.__ghost.open_ghost_manager()),
                              None, 1, 1],
-            'File/Usage': [('Usage', None, _('Usage graph(_A)'), None,
+            'Information': [('Information', None, _('Information(_I)'), None),
+                      None, 1, 1],
+            'Information/Usage': [('Usage', None, _('Usage graph(_A)'), None,
                             '', lambda *a: self.__ghost.show_usage()),
                            None, 1, 1],
-            'File/Version': [('Version', None, _('Version(_V)'), None,
+            'Information/Version': [('Version', None, _('Version(_V)'), None,
                               '', lambda *a: self.__ghost.about()),
                              None, 1, 1],
-            'File/Exit': [('Exit', None, _('Exit(_Q)'), None,
-                           '', lambda *a: self.__ghost.close()),
-                          None, 1, 1],
+            'Close': [('Close', None, _('Close(_W)'), None,
+                      '', lambda *a: self.__ghost.close()),
+                     None, 1, 1],
+            'Quit': [('Quit', None, _('Quit(_Q)'), None,
+                      '', lambda *a: self.__ghost.app.close_all_ghosts()),
+                     None, 1, 1],
             'Change': [('Change', None, _('Change(_G)'), None),
                        None, 1, 1],
             'Summon': [('Summon', None, _('Summon(_X)'), None),
@@ -256,9 +268,11 @@ class Menu:
                 sidebar_width, 0, width, height,
                 0, 0,
                 1.0, 1.0, gtk.gdk.INTERP_BILINEAR, 255)
-            pixbuf = new_pixbuf
             self.__sidebar_width = sidebar_width
-            pixmap, mask = pixbuf.render_pixmap_and_mask(255)
+            pixmap, mask = new_pixbuf.render_pixmap_and_mask(255)
+            del pixbuf
+            del new_pixbuf
+            gc.collect()
         else:
             self.__sidebar_width = 0
         self.__pixmap_with_sidebar = pixmap
@@ -297,8 +311,11 @@ class Menu:
                 sidebar_width, 0, width, height,
                 0, 0,
                 1.0, 1.0, gtk.gdk.INTERP_BILINEAR, 255)
-            pixbuf = new_pixbuf
-            pixmap, mask = pixbuf.render_pixmap_and_mask(255)
+            pixmap, mask = new_pixbuf.render_pixmap_and_mask(255)
+            del pixbuf
+            del new_pixbuf
+            del pixbuf_sidebar
+            gc.collect()
         self.__pixmap_prelight_with_sidebar = pixmap
 
     def __set_mayuna_menu(self, side):
@@ -347,10 +364,10 @@ class Menu:
     def __cut_mnemonic(self, caption):
         return self.__re_mnemonic.sub('', caption)
 
-    __ui = {'File/Update': ([['updatebuttoncaption', 'updatebutton.caption'],
+    __ui = {'Options/Update': ([['updatebuttoncaption', 'updatebutton.caption'],
                              ['updatebuttoncaption', 'updatebutton.caption']],
                             '(_U)', [[],[]]),
-            'File/Vanish': ([['vanishbuttoncaption', 'vanishbutton.caption'],
+            'Options/Vanish': ([['vanishbuttoncaption', 'vanishbutton.caption'],
                              ['vanishbuttoncaption', 'vanishbutton.caption']],
                             '(_F)',
                             [['vanishbuttonvisible', 'vanishbutton.visible'],
@@ -367,7 +384,7 @@ class Menu:
             if key not in self.__menu_list:
                 continue
             if side > 1:
-                if key in ['File/Update', 'File/Vanish']:
+                if key in ['Options/Update', 'Options/Vanish']:
                     name_list = self.__ui[key][0][1] # same as 'kero'
                 elif key == 'Portal':
                     name_list = [] # same as 'kero'
@@ -474,7 +491,7 @@ class Menu:
             if item and changed:
                 self.__menu_list[key][3] = 0 # changed
                 if visible:
-                    if not key.startswith('File/'): # XXX
+                    if not key.startswith('Options/'): # XXX
                         if self.__pixmap_prelight_with_sidebar:
                             style = item.style.copy()
                             style.bg_pixmap[gtk.STATE_PRELIGHT] = self.__pixmap_prelight_with_sidebar
@@ -597,27 +614,16 @@ class Menu:
         self.__menu_list['Recommend'][3] = 1 # changed
 
     def __update_ghost_menu(self, set_list):
-        ##if self.__ghost_list is None:
-        ##    self.__create_ghost_menu(set_list)
-        ##else:
         ghost_menu = gtk.Menu()
         for i in range(len(set_list)):
             if set_list[i] is None:
                 continue
             name = set_list[i]['name']
-            icon_path = set_list[i]['icon']
-            pixbuf = None
-            if icon_path is not None:
-                try:
-                    pixbuf = ninix.pix.create_pixbuf_from_file(icon_path,
-                                                               is_pnr=0)
-                except:
-                    pixbuf = None
+            icon = set_list[i]['icon']
             shell_list = set_list[i]['shell']
-            if pixbuf is not None:
-                pixbuf = pixbuf.scale_simple(16, 16, gtk.gdk.INTERP_BILINEAR)
+            if icon is not None:
                 image = gtk.Image()
-                image.set_from_pixbuf(pixbuf)
+                image.set_from_pixbuf(icon)
                 image.show()
                 item = gtk.ImageMenuItem(name)
                 item.set_image(image)
@@ -651,35 +657,25 @@ class Menu:
         menuitem =  self.ui_manager.get_widget(''.join(('/popup/', 'Change')))
         menuitem.set_submenu(ghost_menu)
         self.__menu_list['Change'][3] = 1
-        self.__ghost_list = set_list
+        ##self.__ghost_list = set_list
 
     def __set_ghost_menu(self):
         ghosts = self.__ghost.app.get_ghost_list() ## FIXME
-        self.__update_ghost_menu(ghosts)
-        self.__update_summon_menu(ghosts)
+        if ghosts != self.__ghost_list or self.__ghost_list is None:
+            self.__update_ghost_menu(ghosts)
+            self.__update_summon_menu(ghosts)
 
     def __update_summon_menu(self, set_list):
-        ##if self.__ghost_list is None:
-        ##    self.__create_ghost_menu(set_list)
-        ##else:
         ghost_menu = gtk.Menu()
         for i in range(len(set_list)):
             if set_list[i] is None:
                 continue
             name = set_list[i]['name']
-            icon_path = set_list[i]['icon']
-            pixbuf = None
-            if icon_path is not None:
-                try:
-                    pixbuf = ninix.pix.create_pixbuf_from_file(icon_path,
-                                                               is_pnr=0)
-                except:
-                    pixbuf = None
+            icon = set_list[i]['icon']
             shell_list = set_list[i]['shell']
-            if pixbuf is not None:
-                pixbuf = pixbuf.scale_simple(16, 16, gtk.gdk.INTERP_BILINEAR)
+            if icon is not None:
                 image = gtk.Image()
-                image.set_from_pixbuf(pixbuf)
+                image.set_from_pixbuf(icon)
                 image.show()
                 item = gtk.ImageMenuItem(name)
                 item.set_image(image)
@@ -710,7 +706,7 @@ class Menu:
         menuitem = self.ui_manager.get_widget(''.join(('/popup/', 'Summon')))
         menuitem.set_submenu(ghost_menu)
         self.__menu_list['Summon'][3] = 1
-        self.__ghost_list = set_list
+        ##self.__ghost_list = set_list
 
     def __update_shell_menu(self, set_list):
         ##if self.__shell_list is None:
