@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 #
-#  Copyright (C) 2004, 2005 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -48,6 +48,7 @@
 
 import os
 import random
+import gc
 
 if 'DISPLAY' in os.environ:
     import gtk
@@ -330,14 +331,15 @@ class Skin:
                 h = max(1, pixbuf.get_height() * self.__scale / 100)
                 pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
             image, mask = pixbuf.render_pixmap_and_mask(255)
+            del pixbuf
+            gc.collect()
         except:
             self.nekoninni.finalize()
             return
         self.w, self.h = image.get_size()
-        self.mask_pixmap = mask
         self.darea.set_size_request(self.w, self.h)
         self.darea.window.set_back_pixmap(image, False)        
-        self.window.shape_combine_mask(self.mask_pixmap, 0, 0)
+        self.window.shape_combine_mask(mask, 0, 0)
 
     def set_position(self, reset=0):
         scrn_w = gtk.gdk.screen_width()
@@ -523,14 +525,15 @@ class Katochan:
                 h = pixbuf.get_height() * self.__scale / 100
                 pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
             image, mask = pixbuf.render_pixmap_and_mask(255)
+            del pixbuf
+            gc.collect()
         except:
             self.nekoninni.finalize()
             return
         self.w, self.h = image.get_size()
-        self.mask_pixmap = mask
         self.darea.set_size_request(self.w, self.h)
         self.darea.window.set_back_pixmap(image, False)
-        self.window.shape_combine_mask(self.mask_pixmap, 0, 0)
+        self.window.shape_combine_mask(mask, 0, 0)
         self.darea.queue_draw_area(0, 0, self.w, self.h)
 
     def load(self, data):
