@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001-2004 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2004, 2005 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -17,6 +17,7 @@ import re
 import sys
 import urllib
 import time
+import gc
 
 from xml.dom import pulldom
 
@@ -497,6 +498,8 @@ class UI:
                 try:
                     pixbuf = ninix.pix.create_pixbuf_from_file(filename)
                     image, mask = pixbuf.render_pixmap_and_mask(255)
+                    del pixbuf
+                    gc.collect()
                 except:
                     darea.window.set_back_pixmap(None, False)
                 else:
