@@ -1,6 +1,6 @@
 # -*- coding: ascii -*-
 #
-#  Copyright (C) 2003-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -13,6 +13,7 @@
 import sys
 import os
 import md5
+import gc
 
 if 'DISPLAY' in os.environ:
     import gtk
@@ -88,16 +89,6 @@ def create_pixbuf_from_DDP_file(path):
     loader.close()
     return pixbuf
 
-def add_alpha_from_PNA_file(pixbuf, path):
-    assert pixbuf.get_has_alpha()
-    pna_pixbuf = gtk.gdk.pixbuf_new_from_file(path)
-    pna_array = pna_pixbuf.get_pixels_array()
-    assert pna_pixbuf.get_bits_per_sample() / 8 == 1 ## FIXME
-    pixbuf_array = pixbuf.get_pixels_array()
-    width = pixbuf.get_width()
-    height = pixbuf.get_height()
-    pixbuf_array[:,:,3] = pna_array[:,:,0]
-
 def create_pixbuf_from_file(path, is_pnr=1, use_pna=0):
     head, tail = os.path.split(path)
     basename, suffix = os.path.splitext(tail)
@@ -127,27 +118,19 @@ def create_pixbuf_from_file(path, is_pnr=1, use_pna=0):
     if use_pna:
         path = os.path.join(head, ''.join((basename, '.pna')))
         if os.path.exists(path):
-            add_alpha_from_PNA_file(pixbuf, path)
+            assert pixbuf.get_has_alpha()
+            pna_pixbuf = gtk.gdk.pixbuf_new_from_file(path)
+            pna_array = pna_pixbuf.get_pixels_array()
+            assert pna_pixbuf.get_bits_per_sample() / 8 == 1 ## FIXME
+            pixbuf_array = pixbuf.get_pixels_array()
+            pixbuf_array[:,:,3] = pna_array[:,:,0]
+            del pna_pixbuf
+            gc.collect()
     return pixbuf
 
 def create_pixmap_from_file(path):
     pixbuf = create_pixbuf_from_file(path)
     pixmap, mask = pixbuf.render_pixmap_and_mask(1)
+    del pixbuf
+    gc.collect()
     return pixmap, mask
-
-def reduce_pixbuf(target_pixbuf, pixbuf, dest_x, dest_y):
-    if pixbuf.get_has_alpha():
-        dest_w = target_pixbuf.get_width()
-        dest_h = target_pixbuf.get_height()
-        target_array = target_pixbuf.get_pixels_array()
-        array = pixbuf.get_pixels_array()
-        w = pixbuf.get_width()
-        h = pixbuf.get_height()
-        for i in range(h):
-            for j in range(w):
-                if array[i][j][3] == 0: # alpha
-                    x = j + dest_x
-                    y = i + dest_y
-                    if 0 <= x < dest_w and 0 <= y < dest_h:
-                        target_array[y][x][3] = 0 # alpha
-    return target_pixbuf
