@@ -11,7 +11,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: script.py,v 1.14 2005/09/07 05:16:46 shy Exp $
+# $Id: script.py,v 1.16 2006/08/26 16:01:18 shy Exp $
 #
 
 import re
@@ -204,6 +204,7 @@ class Parser:
                 self.script.append((SCRIPT_TAG, lexeme, argument))
             elif lexeme.startswith('\\s') or \
                  lexeme.startswith('\\b') or \
+                 lexeme.startswith('\\p') or \
                  lexeme.startswith('\\w'):
                 num = lexeme[2]
                 if lexeme.startswith('\\s') and scope == 1:
@@ -267,6 +268,10 @@ class Parser:
                 else:
                     anchor = None
                     self.script.append((SCRIPT_TAG, lexeme))
+            elif lexeme in ['\\f']:
+                args = [arg[0][1] for arg in \
+                        self.split_params(self.read_sbra_text())]
+                self.script.append((SCRIPT_TAG, lexeme) + tuple(args))
             else:
                 raise self.perror('unknown tag (%s)' % lexeme, skip='length')
         if anchor:
