@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -11,7 +11,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: seriko.py,v 1.25 2006/10/05 15:16:44 shy Exp $
+# $Id: seriko.py,v 1.27 2007/04/10 14:45:38 shy Exp $
 #
 
 import re
@@ -38,23 +38,12 @@ class Controler:
         self.frame = {'step': 0.033, # [sec]
                       'next': 0}
         self.active = []
-        self.__cache = []
         self.__move = None
         self.__dirty = 1
 
-    def cache_is_empty(self):
-        if self.__cache:
-            return False
-        else:
-            return True
-
-    def get_cache(self):
-        assert self.__cache is not None
-        return self.__cache[0][2]
-
     def set_base_id(self, window, surface_id):
         if surface_id == -2:
-            self.terminate()
+            self.terminate(window)
             self.base_id = window.surface_id
         else:
             self.base_id = surface_id
@@ -63,39 +52,6 @@ class Controler:
     def move_surface(self, xoffset, yoffset):
         self.__move = (xoffset, yoffset)
 
-    def draw_overlays(self, window, surface_pixbuf):
-        ##print 'draw_overlays()'
-        if surface_pixbuf is None:
-            return None
-        for pixbuf_id, x, y in self.iter_overlays():
-            try:
-                pixbuf = window.get_pixbuf(pixbuf_id)
-                w = pixbuf.get_width()
-                h = pixbuf.get_height()
-            except:
-                continue
-            # overlay surface pixbuf
-            sw = surface_pixbuf.get_width()
-            sh = surface_pixbuf.get_height()
-            if x + w > sw:
-                w = sw - x
-            if y + h > sh:
-                h = sh - y
-            if x < 0:
-                dest_x = 0
-                w += x
-            else:
-                dest_x = x
-            if y < 0:
-                dest_y = 0
-                h += y
-            else:
-                dest_y = y
-            pixbuf.composite(surface_pixbuf, dest_x, dest_y,
-                             w, h, x, y, 1.0, 1.0,
-                             gtk.gdk.INTERP_BILINEAR, 255)
-        return surface_pixbuf
-
     def append_actor(self, frame, actor):
         self.active.append((frame, actor))
 
@@ -112,7 +68,7 @@ class Controler:
     def update_frame(self, window):
         if self.terminate_flag:
             return False
-        if len(self.__cache) > 1.0 / self.frame['step']: # XXX
+        if len(window.frame_buffer) > 1.0 / self.frame['step']: # XXX
             return True
         self.frame['next'] += self.frame['step']
         frame, actor = self.get_actor_next(window)
@@ -121,7 +77,7 @@ class Controler:
             actor.update(window, frame)
             last_actor = actor
             frame, actor = self.get_actor_next(window)
-        self.update_surface_cache(window)
+        self.update_surface_frame_buffer(window)
         if last_actor is not None and last_actor.exclusive and \
            last_actor.terminate_flag and self.exclusive_actor is None: # XXX
             self.invoke_restart(window)
@@ -135,30 +91,21 @@ class Controler:
                 return self.active.pop(0)
         return None, None
 
-    def update_surface_cache(self, window):
-        if self.__dirty:
-            surface_pixbuf = window.create_surface_pixbuf(self.base_id)
-            surface_pixbuf = self.draw_overlays(window, surface_pixbuf)
-        else:
-            surface_pixbuf = None
-        self.__cache.append((self.frame['next'], self.__move, surface_pixbuf, self.__dirty))
+    def update_surface_frame_buffer(self, window):
+        window.update_frame_buffer(
+            self, self.frame['next'], self.__move, self.__dirty)
         self.__dirty = 0
 
     def update(self, window):
         if self.terminate_flag:
             return False
-        if self.__cache:
-            frame, move, surface, dirty = self.__cache.pop(0)
-            if move is not None:
-                window.move_surface(*move)
-            if dirty:
-                window.update(surface)
+        window.update()
         self.timeout_id = gobject.timeout_add(int(self.frame['step'] * 1000), self.update, window)
         return False
 
     def lock_exclusive(self, window, actor):
         assert self.exclusive_actor is None
-        self.terminate()
+        self.terminate(window)
         self.exclusive_actor = actor
         actor.set_post_proc(self.unlock_exclusive, (window, actor))
         self.update_frame(window) # XXX
@@ -179,9 +126,9 @@ class Controler:
             pass
         self.__dirty = 1
 
-    def add_overlay(self, actor, pixbuf_id, x, y):
+    def add_overlay(self, window, actor, pixbuf_id, x, y):
         if pixbuf_id == '-2':
-            self.terminate()
+            self.terminate(window)
         if pixbuf_id in ['-1', '-2']:
             self.remove_overlay(actor)
             return
@@ -264,7 +211,7 @@ class Controler:
 
     def reset(self, window, surface_id):
         self.queue = []
-        self.terminate()
+        self.terminate(window)
         self.frame['next'] = 0
         self.set_base_id(window, window.surface_id)
         if surface_id in self.seriko:
@@ -286,7 +233,7 @@ class Controler:
         self.invoke_always(window)
         self.timeout_id = gobject.timeout_add(int(self.frame['step'] * 1000), self.update, window)
 
-    def terminate(self):
+    def terminate(self, window):
         self.terminate_flag = 1
         if self.timeout_id is not None:
             gobject.source_remove(self.timeout_id)
@@ -301,7 +248,7 @@ class Controler:
         self.active = []
         self.__move = None
         self.__dirty = 1
-        self.__cache = []
+        window.clear_frame_buffer()
         self.terminate_flag = 0
 
     def iter_overlays(self):
