@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2007 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -17,6 +17,7 @@ import os
 import re
 import urlparse
 import random
+import gc
 
 if 'DISPLAY' in os.environ:
     import gtk
@@ -452,6 +453,8 @@ class Surface:
                     key, method, filename, x, y, w, h)
         if error is not None:
             print error
+            pixbuf_list = None
+            gc.collect()
             pixbuf_list = []
         return pixbuf_list
 
@@ -681,6 +684,8 @@ class Surface:
                 pixbuf = None
         for window in self.window:
             window.window.set_icon(pixbuf)
+        del pixbuf
+        gc.collect()
 
     def __check_mikire_kasanari(self):
         if not self.is_shown(0):
@@ -824,6 +829,7 @@ class SurfaceWindow:
         self.mayuna = mayuna
         self.bind = bind
         self.reset_pixbuf_cache()
+        self.frame_buffer = []
         self.default_id = default_id
         self.debug = debug
         self.__shown = 0
@@ -924,7 +930,7 @@ class SurfaceWindow:
             if aliases:
                 surface_id = random.choice(aliases)
         if surface_id == '-2':
-            self.seriko.terminate()
+            self.seriko.terminate(self)
         if surface_id in ['-1', '-2']:
             pass
         elif surface_id not in self.pixbuf:
@@ -958,9 +964,8 @@ class SurfaceWindow:
         self.set_position(x, y)
         self.sakura.ghost.notify_surface_change(self.side) ## FIXME
 
-    def compose_surface(self, surface_pixbuf, mayuna, done):
-        for pattern in mayuna.patterns:
-            surface, interval, method, args = pattern
+    def iter_mayuna(self, surface_width, surface_height, mayuna, done):
+        for surface, interval, method, args in mayuna.patterns:
             if method in ['bind', 'add']:
                 if surface in self.pixbuf:
                     x, y = args
@@ -968,12 +973,10 @@ class SurfaceWindow:
                     w = pixbuf.get_width()
                     h = pixbuf.get_height()
                     # overlay surface pixbuf
-                    sw = surface_pixbuf.get_width()
-                    sh = surface_pixbuf.get_height()
-                    if x + w > sw:
-                        w = sw - x
-                    if y + h > sh:
-                        h = sh - y
+                    if x + w > surface_width:
+                        w = surface_width - x
+                    if y + h > surface_height:
+                        h = surface_height - y
                     if x < 0:
                         dest_x = 0
                         w += x
@@ -984,17 +987,15 @@ class SurfaceWindow:
                         h += y
                     else:
                         dest_y = y
-                    pixbuf.composite(surface_pixbuf, dest_x, dest_y,
-                                     w, h, x, y, 1.0, 1.0,
-                                     gtk.gdk.INTERP_BILINEAR, 255)
+                    yield method, pixbuf, dest_x, dest_y, w, h, x, y
             elif method == 'reduce':
                 if surface in self.pixbuf:
-                    x, y = args
+                    dest_x, dest_y = args
                     pixbuf = self.get_pixbuf(surface)
                     w = pixbuf.get_width()
                     h = pixbuf.get_height()
-                    surface_pixbuf = pix.reduce_pixbuf(
-                        surface_pixbuf, pixbuf, x, y)
+                    x = y = 0 # XXX
+                    yield method, pixbuf, dest_x, dest_y, w, h, x, y
             elif method == 'insert':
                 index = args[0]
                 for actor in self.mayuna[self.surface_id]:
@@ -1003,15 +1004,17 @@ class SurfaceWindow:
                         if actor_id in self.bind and self.bind[actor_id][1] and \
                            actor_id not in done:
                             done.append(actor_id)
-                            surface_pixbuf = self.compose_surface(
-                                surface_pixbuf, actor, done)
+                            for result in self.iter_mayuna(surface_width, surface_height, actor, done):
+                                yield result
                         else:
                             break
             else:
                 raise RuntimeError, 'should not reach here'
-        return surface_pixbuf
 
     def reset_pixbuf_cache(self):
+        self.pixbuf_cache = None
+        self.mayuna_cache = None
+        gc.collect()
         self.pixbuf_cache = {}
         self.mayuna_cache = {}
 
@@ -1066,7 +1069,10 @@ class SurfaceWindow:
                 dest_y = y
             overlay.composite(pixbuf, dest_x, dest_y,
                               w, h, x, y, 1.0, 1.0,
-                              gtk.gdk.INTERP_BILINEAR, 255)        
+                              gtk.gdk.INTERP_BILINEAR, 255)
+            del overlay
+        ##else:
+        ##    gc.collect() # XXX
         return pixbuf
 
     def get_pixbuf(self, pixbuf_id):
@@ -1106,16 +1112,83 @@ class SurfaceWindow:
                     if actor_id in self.bind and self.bind[actor_id][1] and \
                        actor_id not in done:
                         done.append(actor_id)
-                        surface_pixbuf = self.compose_surface(
-                            surface_pixbuf, actor, done)
+                        #surface_pixbuf = self.compose_surface(
+                        #    surface_pixbuf, actor, done)
+                        surface_width = surface_pixbuf.get_width()
+                        surface_height = surface_pixbuf.get_height()
+                        for method, pixbuf, dest_x, dest_y, w, h, x, y in self.iter_mayuna(surface_width, surface_height, actor, done):
+                            if method in ['bind', 'add']:
+                                pixbuf.composite(surface_pixbuf, dest_x, dest_y,
+                                                 w, h, x, y, 1.0, 1.0,
+                                                 gtk.gdk.INTERP_BILINEAR, 255)
+                            elif method == 'reduce':
+                                if pixbuf.get_has_alpha():
+                                    dest_w = surface_width
+                                    dest_h = surface_height
+                                    surface_array = surface_pixbuf.get_pixels_array()
+                                    array = pixbuf.get_pixels_array()
+                                    for i in range(h):
+                                        for j in range(w):
+                                            if array[i][j][3] == 0: # alpha
+                                                x = j + dest_x
+                                                y = i + dest_y
+                                                if 0 <= x < dest_w and 0 <= y < dest_h:
+                                                    surface_array[y][x][3] = 0 # alpha
+                            else:
+                                raise RuntimeError, 'should not reach here'
                 self.mayuna_cache[surface_id] = surface_pixbuf.copy()
         else:
             surface_pixbuf =self.get_pixbuf(surface_id)
         return surface_pixbuf
 
-    def update(self, cache=None):
-        if cache is not None:
-            surface_pixbuf = cache
+    def clear_frame_buffer(self):
+        self.frame_buffer = []
+
+    def update_frame_buffer(self, seriko, frame, move, dirty):
+        if dirty:
+            surface_pixbuf = self.create_surface_pixbuf(seriko.base_id)
+            if surface_pixbuf is not None:
+                # draw overlays
+                for pixbuf_id, x, y in seriko.iter_overlays():
+                    try:
+                        pixbuf = self.get_pixbuf(pixbuf_id)
+                        w = pixbuf.get_width()
+                        h = pixbuf.get_height()
+                    except:
+                        continue
+                    # overlay surface pixbuf
+                    sw = surface_pixbuf.get_width()
+                    sh = surface_pixbuf.get_height()
+                    if x + w > sw:
+                        w = sw - x
+                    if y + h > sh:
+                        h = sh - y
+                    if x < 0:
+                        dest_x = 0
+                        w += x
+                    else:
+                        dest_x = x
+                    if y < 0:
+                        dest_y = 0
+                        h += y
+                    else:
+                        dest_y = y
+                    pixbuf.composite(surface_pixbuf, dest_x, dest_y,
+                                     w, h, x, y, 1.0, 1.0,
+                                     gtk.gdk.INTERP_BILINEAR, 255)
+                    del pixbuf
+                ##gc.collect() # XXX
+        else:
+            surface_pixbuf = None
+        self.frame_buffer.append((frame, move, surface_pixbuf, dirty))
+
+    def update(self):
+        if len(self.frame_buffer) > 0:
+            frame, move, surface_pixbuf, dirty = self.frame_buffer.pop(0)
+            if move is not None:
+                self.move_surface(*move)
+            if not dirty:
+                return
         else:
             surface_pixbuf = self.create_surface_pixbuf()
         if self.__scale != 100:
@@ -1123,8 +1196,8 @@ class SurfaceWindow:
             h = max(1, surface_pixbuf.get_height() * self.__scale / 100)
             surface_pixbuf = surface_pixbuf.scale_simple(
                 w, h, gtk.gdk.INTERP_BILINEAR)
-        surface_pixmap, self.mask_pixmap = \
-                        surface_pixbuf.render_pixmap_and_mask(1)
+        surface_pixmap, mask_pixmap = surface_pixbuf.render_pixmap_and_mask(1)
+        self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.current_surface_pixbuf = surface_pixbuf
         self.darea.queue_draw()
 
@@ -1135,7 +1208,6 @@ class SurfaceWindow:
         cr.paint()
         cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
         cr.paint_with_alpha(self.__alpha_channel)
-        self.window.shape_combine_mask(self.mask_pixmap, 0, 0)
         if self.debug & 4096:
             self.draw_region()
 
@@ -1143,7 +1215,7 @@ class SurfaceWindow:
         self.seriko.remove_overlay(actor)
 
     def add_overlay(self, actor, pixbuf_id, x, y):
-        self.seriko.add_overlay(actor, pixbuf_id, x, y)
+        self.seriko.add_overlay(self, actor, pixbuf_id, x, y)
 
     def __move(self, xoffset=0, yoffset=0):
         x, y = self.get_position()
@@ -1237,7 +1309,7 @@ class SurfaceWindow:
         self.reset_pixbuf_cache()
         for tag in self.callbacks:
             self.darea.disconnect(tag)
-        self.seriko.terminate()
+        self.seriko.terminate(self)
         self.window.remove(self.darea)
         self.darea.destroy()
         self.window.destroy()
